package typerep

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Variant_and_record_intf.MSource

Parameters

module X : sig ... end

Signature

Sourcemodule Tag_internal : sig ... end
Sourcemodule Tag : sig ... end

Witness of a tag, that is an item in a variant type, also called an "applied variant Constructor"

Sourcemodule Variant_internal : sig ... end
Sourcemodule Variant : sig ... end
Sourcemodule Field_internal : sig ... end
Sourcemodule Field : sig ... end

Witness of a field, that is an item in a record type. The first parameter is the record type, the second is the type of the field. Example:

Sourcemodule Record_internal : sig ... end
Sourcemodule Record : sig ... end