package typerep

  1. Overview
  2. Docs
Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Module Typerep_lib.Typerep_objSource

Sourceval repr_of_poly_variant : [> ] -> int

some utils related to the runtime of ocaml, used both at compile time (camlp4) and runtime. to be considered the same way than Obj (internal, unsafe, etc.)

Sourceval hash_variant : string -> int
Sourceval double_array_value : unit -> 'a
Sourceval has_double_array_tag : 'a -> bool